const system = require("../../../system");
const ServiceBase = require("../../sve.base");
const settings = require("../../../../config/settings");

class ProductService extends ServiceBase {
  constructor() {
    super("dbproduct", ServiceBase.getDaoName(ProductService));
    this.productpriceDao = system.getObject("db.dbproduct.productpriceDao");
    this.productinterfaceDao = system.getObject("db.dbproduct.productinterfaceDao");
  }
  /**
   * 通过产品类别编码路径获取产品列表
   * @param {*} actionBody pathCode 为类别编码路径，一级类下产品列表 sbfu，二级类下产品列表sbfu/sbzc
   * @param {*} appInfo 应用信息
   */
  async getProductList(actionBody, appInfo) {
    var sql = "select * from v_product where uapp_id=" + appInfo.uapp_id + " and path_code like '" + actionBody.pathCode + "%'";
    var list = await this.customQuery(sql);
    return system.getResultSuccess(list);
  }
  /**
   * 通过渠道产品编码获取产品详情
   * @param {*} actionBody channelItemCode 渠道产品编码
   * @param {*} appInfo 应用信息
   */
  async getProductDetail(actionBody, appInfo) {
    var item = await this.dao.getItemByChannelItemCode(actionBody.channelItemCode, appInfo.uapp_id);
    if (!item) {
      return system.getResult(null, "product item is data empty ！");
    }
    if (item.is_enabled != 1) {
      return system.getResult(null, "product to item is Disable ！");
    }
    var pList = await this.productpriceDao.model.findAll({
      where: { product_id: item.id },
      attributes: ["id", "pay_code", "price", "supply_price", "service_charge", "public_expense", "is_default", "price_type", "price_type_name",
        "sort", "price_desc", "min_qty", "max_qty", "service_code"
      ],
      raw: true
    });
    if (!pList || pList.length == 0) {
      return system.getResult(null, "product to price is data empty ！");
    }
    item.price_list = pList;
    return system.getResultSuccess(item);
  }
  /**
   * 获取产品接口信息
   * @param {*} actionBody productId产品id
   */
  async getProductInterface(actionBody) {
    var list = await this.productinterfaceDao.getListByProductPriceId(actionBody.productPriceId);
    // if (!item) {
    //   return system.getResult(null, "product to interface is data empty ！");
    // }
    return system.getResultSuccess(list);
  }

}
module.exports = ProductService;
